Imports System.Messaging
Imports System.Security

Public Class FileSystemAccessControlEntry
    Inherits AccessControlEntry

    Public Sub New()
        MyBase.New()
    End Sub

    Public Sub New(ByVal id As Trustee)
        MyBase.New(id)
    End Sub

    Public Sub New(ByVal id As Trustee, ByVal rights As GenericAccessRights, ByVal stdrights As StandardAccessRights, ByVal acetype As AccessControlEntryType)
        MyBase.New(id, rights, stdrights, acetype)
    End Sub

    Public Property FileAccessRights() As FileAccessRights
        Get
            Return CType(CustomAccessRights, FileAccessRights)
        End Get
        Set(ByVal Value As FileAccessRights)
            customaccessrights = Value
        End Set
    End Property

End Class

Public Enum FileAccessRights
    FileRead = &H1
    FileWrite = &H2
    FileAppend = &H4
    FileReadProperties = &H8
    FileWriteProperties = &H10
    FileExecute = &H20
    FileReadAttributes = &H80
    FileWriteAttributes = &H100
    DirectoryList = &H1
    DirectoryAddFile = &H2
    DirectoryAddSubdirectory = &H4
    DirectoryDeleteChild = &H40
    DirectoryTraverse = &H20
End Enum

Public Enum FileSystemInheritanceType
    [Default] = 0
    AllowInheritance = 1
    BlockInheritance = 2
End Enum


Public Class FileSystemAccessControlList
    Inherits AccessControlList

    Private Const MAXIMUM_ALLOWED As Integer = &H2000000
    Private Const GENERIC_ALL As Integer = &H10000000
    Private Const SE_FILE_OBJECT As Integer = 1
    Private Const DACL_SECURITY_INFORMATION As Integer = &H4
    Private Const PROTECTED_DACL_SECURITY_INFORMATION As Integer = &H80000000
    Private Const UNPROTECTED_DACL_SECURITY_INFORMATION As Integer = &H20000000

    Private Declare Auto Function GetNamedSecurityInfo Lib "advapi32.dll" (ByVal filename As String, _
      ByVal ObjectType As Integer, ByVal SecurityInfo As Integer, ByRef ppsidOwner As Integer, _
      ByRef ppsidGroup As Integer, ByRef ppDacl As Integer, ByRef ppSacl As Integer, ByRef ppSecurityDescriptor As Integer) As Integer

    Private Declare Auto Function SetNamedSecurityInfo Lib "advapi32.dll" (ByVal filename As String, _
      ByVal ObjectType As Integer, ByVal SecurityInfo As Integer, ByVal psidOwner As Integer, _
      ByVal psidGroup As Integer, ByVal pDacl As Integer, ByVal pSacl As Integer) As Integer

    Private m_InheritanceType As FileSystemInheritanceType
    Private m_CombineExisting As Boolean

    Public Property InheritanceSettings() As FileSystemInheritanceType
        Get
            Return m_InheritanceType
        End Get
        Set(ByVal Value As FileSystemInheritanceType)
            m_InheritanceType = Value
        End Set
    End Property

    Public Property CombineExisting() As Boolean
        Get
            Return m_CombineExisting
        End Get
        Set(ByVal Value As Boolean)
            m_CombineExisting = Value
        End Set
    End Property

    Public Sub SetFileSecurity(ByVal FileOrDirectory As String)

        ' SecurityInfo is the flag that specifies what type of security settings to set/retrieve
        ' Default is DACL, but we add the inheritance setting if a non-default is requested
        Dim SecurityInfo As Integer = DACL_SECURITY_INFORMATION
        Select Case m_InheritanceType
            Case FileSystemInheritanceType.AllowInheritance
                SecurityInfo = SecurityInfo Or UNPROTECTED_DACL_SECURITY_INFORMATION
            Case FileSystemInheritanceType.BlockInheritance
                SecurityInfo = SecurityInfo Or PROTECTED_DACL_SECURITY_INFORMATION
        End Select

        Dim fi As New IO.FileInfo(FileOrDirectory)
        Dim fullpath As String

        ' First see if it is a file that exists
        If Not fi.Exists Then
            ' On failure, check if it's a directory.
            Dim di As New IO.DirectoryInfo(FileOrDirectory)
            If Not di.Exists Then Throw New IO.FileNotFoundException(FileOrDirectory)
            fullpath = di.FullName
        Else
            fullpath = fi.FullName
        End If

        ' Demand the appropriate file access permission
        Dim fileperm As New Permissions.FileIOPermission(Security.Permissions.FileIOPermissionAccess.AllAccess, fullpath)
        fileperm.Demand()

        ' If allowed Then assert unmanaged code permission and reflection permission
        Dim useperm1 As New Permissions.SecurityPermission(Security.Permissions.SecurityPermissionFlag.UnmanagedCode)
        Dim useperm2 As New Permissions.ReflectionPermission(Security.Permissions.ReflectionPermissionFlag.MemberAccess)
        Dim usepermlist As New PermissionSet(Security.Permissions.PermissionState.None)
        usepermlist.AddPermission(useperm1)
        usepermlist.AddPermission(useperm2)
        usepermlist.Assert()

        Try

            Dim siresult As Integer
            Dim oldACL As Integer
            ' If we're combining existing ACL entries, retrieve the existing ACL entries
            If m_CombineExisting Then
                siresult = GetNamedSecurityInfo(fullpath, SE_FILE_OBJECT, DACL_SECURITY_INFORMATION, 0, 0, oldACL, 0, 0)
                If siresult <> 0 Then
                    ' If it fails, throw the appropriate Win32 exception
                    Throw New ComponentModel.Win32Exception(Err.LastDllError)
                End If
            End If
            Dim newACL As Integer

            ' Now call the private MakeAcl method to get a new ACL that optionally merges the existing one.
            Dim newActlPtr As IntPtr = CType(Me.GetType.BaseType.InvokeMember("MakeAcl", Reflection.BindingFlags.InvokeMethod Or Reflection.BindingFlags.Instance Or Reflection.BindingFlags.NonPublic, Nothing, Me, New Object() {New IntPtr(oldACL)}), IntPtr)
            newACL = newActlPtr.ToInt32


            ' And finally, set the new ACL into the file object
            siresult = SetNamedSecurityInfo(fullpath, SE_FILE_OBJECT, SecurityInfo, 0, 0, newACL, 0)

            ' Free the ACL
            Me.GetType.BaseType.InvokeMember("FreeAcl", Reflection.BindingFlags.InvokeMethod Or Reflection.BindingFlags.Static Or Reflection.BindingFlags.NonPublic, Nothing, Me, New Object() {New IntPtr(newACL)})

            If siresult <> 0 Then
                Throw New ComponentModel.Win32Exception(Err.LastDllError)
            End If

        Catch ex As Exception
            Throw ex
        End Try
    End Sub
End Class